



ls
IF %ERRORLEVEL% NEQ 0 exit /B 1
python -c "import os; print ('TESTING IN THIS DIRECTORY' , os.getcwd())"
IF %ERRORLEVEL% NEQ 0 exit /B 1
python -m unittest discover -s Wrappers/Python/test -v
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
